/******************************************************************************
 * FileName:        PIC32_SPI.h
 * Dependencies:    GenericTypeDefs.h
 * Processor:       PIC32
 * Compiler:        CX32
 * Company:         McGee Engineering
 * Author:          Thomas McGee
 * Created:         June 13, 2014, 2:49 PM
*****************************************************************************/

#include <GenericTypeDefs.h>
#include <p32xxxx.h>
#include <stdio.h>
#include <plib.h>            // Include to use PIC32 peripheral libraries
#include <stdint.h>          // For uint32_t definition

/*********************************************************************/
/******************* Pin and Register Definitions ********************/
/*********************************************************************/

#define GetSystemClock()        80000000ul
#define	GetPeripheralClock()    (GetSystemClock()/(1 << OSCCONbits.PBDIV))
#define TICKS_PER_SECOND        ((unsigned long) GetSystemClock()/256)

#define ADDRESS_BUSS_TRIS   mPORTASetPinsDigitalOut(0x00BF)     //Address and parallel bus controls

#define GRAPHICS_BUS_TRIS   mPORTDSetPinsDigitalOut(0x00FF);\
                            mPORTESetPinsDigitalOut(0x00FF)     //Configure PortD as output for low byte (0:7)
                                                                //and PortE as output for High byte (8:15)
//Masks for Parallel communications
#define SEL_RST     mPORTAClearBits(BIT_7)              //Set ReSeT flag.
#define DES_RST     mPORTASetBits(BIT_7)                //clear ReSeT flag.
#define START_WR    mPORTAClearBits(BIT_5)              //Start Graphics Processor Write.
#define END_WR      mPORTASetBits(BIT_5)                //End Graphics Processor Write.
#define SEL_CM      mPORTAClearBits(BIT_4)              //Set Graphics Processor for Command.
#define SEL_DT      mPORTASetBits(BIT_4)                //Set Graphics Processor for Data.

//LCD driver chip
#define SEL_GP      mPORTAClearBits(0b00001111)         //GP address = 0bxxxx0000.

//Touch-Screen controls
#define SEL_TS      mPORTASetBits(0b00001111);\
                    mPORTAClearBits(0b00001110)         //TS address = 0bxxxx0001

#define TS_TD_TRIS  mPORTASetPinsDigitalIn(BIT_14)       //PenIRQ TRIS bit. CN9/RG7(J4.10) MAX#29
#define TS_IRQ      mPORTAReadBits(BIT_14)              //PenIRQ Input bit. CN9/RG7(J4.10) MAX#29
                 
#define DESELECT    mPORTASetBits(0b00001111)           //Deselect.

#define LD5_TRIS    mPORTCSetPinsDigitalOut(BIT_1)      //Configure pin to drive LD5. 
#define LD5_INV     mPORTCToggleBits(BIT_1)             //Toggle LD5. (pulse)
#define LD5_ON      mPORTCSetBits(BIT_1)
#define LD5_OFF     mPORTCClearBits(BIT_1)

/*
 * MODE      CKP     CKE
 * 0        0       1
 * 1        0       0
 * 2        1       1
 * 3        1       0
 */

//SPI Configuration
#define SYNC_MODE_FAST      1000000L                    //Sets SPI module to 1Mbits/s
#define SYNC_MODE_SLOW      250000L                     //Sets SPI module to .25Mbits/s
#define SPI_MASTER          SPI_OPEN_MSTEN
#define SPI_MODE8           0x0000

#define CKP_HIGH            SPI_OPEN_CKP_HIGH
#define CKP_LOW             0x0000
#define CKE_HIGH            SPI_OPEN_CKE_REV
#define CKE_LOW             0x0000

// Registers for the SPI module you want to use
#define SPIBUF          SPI4BUF                     //Description: The SPI Buffer
#define SPISTAT_RBF     SPI4STATbits.SPIRBF         //Description: The receive buffer full bit in the SPI status register

#define TS_DB_ENTRY     3125   // Delay 10mS
#define TS_DB_EXIT      3125   // Delay 10mS
#define TS_POWER        3      //Power of two used.
#define TS_SAMPLES      8      //Set to(2 ^ TS_POWER)

#define X_Slope         1       //0.2136    Set to 1 for calibration
#define X_Offset	0       //-34.78    Set to 0 for calibration
#define Y_Slope         1       //0.1363    Set to 1 for calibration
#define Y_Offset	0       //-28.04    Set to 0 for calibration

/******************************************************************************
 * Global Variables
 *****************************************************************************/

UINT8 ForColorH, ForColorL;
UINT8 BackColorH, BackColorL;

UINT16 TP_X ,TP_Y;   // Touch Screeen

#define clrXY           setXY(0,0,799,479)
#define fontbyte(x)     cfont.font[x]

/******************************************************************************/
/* User Type Definitions                                                       /
/******************************************************************************/
    struct              //Structure for font data
    {
        char* font;
        char x_size;
        char y_size;
        char offset;
        char numchars;
    }cfont;

   typedef union 		//Union for long, int and byte
    {
        long Long;
        struct {int  int0; int int1;};
        struct {char  b0; char  b1; char  b2; char  b3;};
    }bytelong;

    union
    {
        struct
        {
            unsigned TS        :1;   // xxxxxxx1 Touch Screed has been trigered
            unsigned SD_Card   :1;   // xxxxxx1x SDCard has changed
            unsigned WIFI      :1;   // xxxxx1xx WFI request
            unsigned EMP0      :1;   // xxxx1xxx
            unsigned EMP1      :1;   // xxx1xxxx
            unsigned EMP2      :1;   // xx1xxxxx
            unsigned EMP3      :1;   // x1xxxxxx
            unsigned EMP4      :1;   // 1xxxxxxx
        };
        unsigned char sr_byte;           // Set all flages with byte
    } ServiceRequests;
/******************************************************************************
 * Prototypes
 *****************************************************************************/
void Init_buss( unsigned char Mode,long bit_rate);
void modeSPI( unsigned char Mode,long bit_rate);
void comWritPort(char H_byte,char L_byte);
char comSPI( char data_out );


void TickInit(void);
unsigned long TickGet(char mode);
void delayCT(unsigned long Tics);
void delayMS(unsigned long milliS);

void LCD_Init();
void setFont(uint8_t* font);
void setForColor(UINT8 r, UINT8 g, UINT8 b);
void setBackColor(UINT8 r, UINT8 g, UINT8 b);
void clrScr();
void setXY(UINT16 x1, UINT16 y1, UINT16 x2, UINT16 y2);

    // Graphics Functions
void fillScr(UINT8 r, UINT8 g, UINT8 b);
void drawCircle(int x, int y, int radius, UINT8 w);
void printChar(UINT8 c, int x, int y);
void printString(UINT8 *st, UINT16 x, UINT16 y);

void MainScreen(void);
void srTouchScreen(void);