/******************************************************************************
 *
 *                  Tick Manager for Timekeeping
 *
 ******************************************************************************
 * FileName:        Tick.c
 * Dependencies:    Timer 1 
 * Processor:       PIC32
 * Compiler:        Microchip CX32 v1.10b or higher
 *										
 * Company:         Microchip Technology, Inc.
 *
******************************************************************************/
#include "SystemConfig.h"

/******************************************************************************
* Defines
******************************************************************************/

/******************************************************************************
* Local Global Variable
******************************************************************************/
static volatile unsigned long InternalTicks = 0;
// Internal counter to store Ticks.  This variable is incremented in an ISR and
// therefore must be marked volatile to prevent the compiler optimizer from
// reordering code to use this value in the main context while interrupts are
// disabled.



/******************************************************************************
*  Function Name:  void TickInit(void)
*  Return Value:   void
*  Parameters:     void
*  Description:    Configures the Tick module and Timer1.
******************************************************************************/
void TickInit(void)
{
    //configure Timer 1 using internal clock, 1:256 prescale
    OpenTimer1(T1_ON | T1_SOURCE_INT | T1_PS_1_256, 0xFFFF);

    // set up the timer interrupt with a priority of 2
    ConfigIntTimer1(T1_INT_ON | T1_INT_PRIOR_2);
}

/******************************************************************************
*  Function Name:  void _ISR _T1Interrupt(void)
*  Return Value:   none
*  Parameters:     none
*  Description:    Updates the tick value when timer1 interrupt occurs.
******************************************************************************/
void __ISR(_TIMER_1_VECTOR, IPL2AUTO) _T1Interrupt(void)
{
    mT1ClearIntFlag();      //Clear interrupt flag
    InternalTicks++;        // Increment internal high tick counter
}

/******************************************************************************
*  Function Name:  static void GetTickCopy(void)
*  Return Value:   void
*  Parameters:     void
*  Description:    This function performs an interrupt-safe and synchronized
*                  read of the 48-bit Tick value.
******************************************************************************/

unsigned long TickGet(char mode)
{
    DWORD dwTempTicks;
    BYTE vTickReading[6];
    // Perform an Interrupt safe and synchronized read of the 48-bit
    // tick value

    do
    {
	IEC0SET = _IEC0_T1IE_MASK;	// Enable interrupt
	Nop();
	IEC0CLR = _IEC0_T1IE_MASK;	// Disable interrupt

	// Get low 2 bytes
	((volatile WORD*)vTickReading)[0] = TMR1;

	// Correct corner case where interrupt increments byte[4+] but
	// TMR1 hasn't rolled over to 0x0000 yet
	dwTempTicks = InternalTicks;
     } while(IFS0bits.T1IF);

     IEC0SET = _IEC0_T1IE_MASK;		// Enable interrupt


     switch (mode)
     {
         case 0:
            vTickReading[2] = ((BYTE*)&dwTempTicks)[0];
            vTickReading[3] = ((BYTE*)&dwTempTicks)[1];
            break;

         case 1:
            vTickReading[0] = vTickReading[1];
            vTickReading[1] = ((BYTE*)&dwTempTicks)[0];
            vTickReading[2] = ((BYTE*)&dwTempTicks)[1];
            vTickReading[3] = ((BYTE*)&dwTempTicks)[2];
            break;

         case 2:
            vTickReading[0] = ((BYTE*)&dwTempTicks)[0];
            vTickReading[1] = ((BYTE*)&dwTempTicks)[1];
            vTickReading[2] = ((BYTE*)&dwTempTicks)[2];
            vTickReading[3] = ((BYTE*)&dwTempTicks)[3];
            break;
         default:
             break;
     }

        return *((DWORD*)&vTickReading[0]);
   
}

/******************************************************************************
*  Function Name:  delayCT()
*  Return Value:   void
*  Parameters:     void
*  Description:    counts number of cycles of timer1.
******************************************************************************/
void delayCT(unsigned long Tics)
{
    unsigned long temp;
    
   // one timer1 tic = 3.2 micro seconds
    temp = TickGet(0);
    while((TickGet(0) - temp < Tics));
}

/******************************************************************************
*  Function Name:  delayMS()
*  Return Value:   UINT32 milliS
*  Parameters:     void
*  Description:    Delay for number of milli seconds with timer1.
******************************************************************************/
void delayMS(unsigned long milliS)
{
    unsigned long temp;
    // one timer1 tic = 3.2 micro seconds
    // one Milli second = 312.5 tics
    milliS =  (unsigned long)((float)milliS * (float)312.5);
    temp = TickGet(0);
    while((TickGet(0) - temp < milliS));
}


